@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# cython: language_level=3
+# cython: language_level=3, always_allow_keywords=True
 
-## Copyright 2004-2013 by LivingLogic AG, Bayreuth/Germany.
-## Copyright 2004-2013 by Walter Dörwald
+## Copyright 2004-2019 by LivingLogic AG, Bayreuth/Germany
+## Copyright 2004-2019 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
@@ -16,8 +16,6 @@
 
 import colorsys
 
-from ll import ul4c
-
 
 __docformat__ = "reStructuredText"
 
@@ -25,8 +23,10 @@ __docformat__ = "reStructuredText"
 class Color(tuple):
 	"""
 	A :class:`Color` object represents a color with red, green and blue
-	components.
+	components and transparency.
 	"""
+	ul4attrs = {"r", "g", "b", "a", "hsv", "hsva", "hls", "hlsa", "lum", "witha", "withlum", "abslum", "rellum"}
+
 	def __new__(cls, r=0x0, g=0x0, b=0x0, a=0xff):
 		"""
 		Create a :class:`Color` with the 8 bit red, green, blue and alpha
@@ -37,15 +37,18 @@ class Color(tuple):
 
 	@classmethod
 	def fromrepr(cls, s):
-		if len(s) == 9:
-			return cls(int(s[1:3], 16), int(s[3:5], 16), int(s[5:7], 16), int(s[7:], 16))
-		elif len(s) == 7:
-			return cls(int(s[1:3], 16), int(s[3:5], 16), int(s[5:], 16))
-		elif len(s) == 5:
-			return cls(17*int(s[1], 16), 17*int(s[2], 16), 17*int(s[3], 16), 17*int(s[4], 16))
-		elif len(s) == 4:
-			return cls(17*int(s[1], 16), 17*int(s[2], 16), 17*int(s[3], 16))
-		raise ValueError("can't interpret {} as color repr value".format(s))
+		try:
+			if len(s) == 9:
+				return cls(int(s[1:3], 16), int(s[3:5], 16), int(s[5:7], 16), int(s[7:], 16))
+			elif len(s) == 7:
+				return cls(int(s[1:3], 16), int(s[3:5], 16), int(s[5:], 16))
+			elif len(s) == 5:
+				return cls(17*int(s[1], 16), 17*int(s[2], 16), 17*int(s[3], 16), 17*int(s[4], 16))
+			elif len(s) == 4:
+				return cls(17*int(s[1], 16), 17*int(s[2], 16), 17*int(s[3], 16))
+		except ValueError:
+			pass
+		raise ValueError(f"can't interpret {s!r} as color repr value")
 
 	@classmethod
 	def fromcss(cls, s):
@@ -86,7 +89,7 @@ class Color(tuple):
 			return cls(*channels)
 		elif s in csscolors:
 			return csscolors[s]
-		raise ValueError("can't interpret {} as css value".format(s))
+		raise ValueError(f"can't interpret {s!r} as css value")
 
 	@classmethod
 	def fromrgb(cls, r, g, b, a=1.0):
@@ -121,44 +124,40 @@ class Color(tuple):
 
 	def __repr__(self):
 		if self[3] != 0xff:
-			return "Color({:#04x}, {:#04x}, {:#04x}, {:#04x})".format(*self)
+			return f"Color({self[0]:#04x}, {self[1]:#04x}, {self[2]:#04x}, {self[3]:#04x})"
 		else:
-			return "Color({:#04x}, {:#04x}, {:#04x})".format(*self)
+			return f"Color({self[0]:#04x}, {self[1]:#04x}, {self[2]:#04x})"
 
 	def __str__(self):
 		"""
 		:obj:`self` formatted as a CSS color string.
 		"""
 		if self[3] != 0xff:
-			return "rgba({},{},{},{:.3f})".format(self[0], self[1], self[2], self[3]/255.)
+			return f"rgba({self[0]},{self[1]},{self[2]},{self[3]/255.:.3f})"
 		else:
-			s = "#{:02x}{:02x}{:02x}".format(self[0], self[1], self[2])
-			if s[1]==s[2] and s[3]==s[4] and s[5]==s[6]:
-				s = "#{}{}{}".format(s[1], s[3], s[5])
+			s = f"#{self[0]:02x}{self[1]:02x}{self[2]:02x}"
+			if s[1] == s[2] and s[3] == s[4] and s[5] == s[6]:
+				s = f"#{s[1]}{s[3]}{s[5]}"
 		return s
 
-	@ul4c.expose_method
 	def r(self):
 		"""
 		The red value as an int between 0 and 255.
 		"""
 		return self[0]
 
-	@ul4c.expose_method
 	def g(self):
 		"""
 		The green value as an int between 0 and 255.
 		"""
 		return self[1]
 
-	@ul4c.expose_method
 	def b(self):
 		"""
 		The blue value as an int between 0 and 255.
 		"""
 		return self[2]
 
-	@ul4c.expose_method
 	def a(self):
 		"""
 		The alpha value as an int between 0 and 255.
@@ -179,39 +178,34 @@ class Color(tuple):
 		"""
 		return (self[0]/255., self[1]/255., self[2]/255., self[3]/255.)
 
-	@ul4c.expose_method
 	def hsv(self):
 		"""
-		:obj:`self` as a HSV ("hue, saturation, value") triple.
+		:obj:`self` as a HSV tuple ("hue, saturation, value").
 		All three values are between 0.0 and 1.0.
 		"""
 		return colorsys.rgb_to_hsv(self[0]/255., self[1]/255., self[2]/255.)
 
-	@ul4c.expose_method
 	def hsva(self):
 		"""
-		:obj:`self` as a HSV+alpha ("hue, saturation, value, alpha") tuple.
+		:obj:`self` as a HSV+alpha tuple ("hue, saturation, value, alpha").
 		All four values are between 0.0 and 1.0.
 		"""
 		return self.hsv() + (self[3]/255.,)
 
-	@ul4c.expose_method
 	def hls(self):
 		"""
-		:obj:`self` as a HLS ("hue, luminance, saturation") triple. All three
+		:obj:`self` as a HLS tuple ("hue, luminance, saturation"). All three
 		values are between 0.0 and 1.0.
 		"""
 		return colorsys.rgb_to_hls(self[0]/255., self[1]/255., self[2]/255.)
 
-	@ul4c.expose_method
 	def hlsa(self):
 		"""
-		:obj:`self` as a HLS+alpha ("hue, luminance, saturation, alpha") tuple.
+		:obj:`self` as a HLS+alpha tuple ("hue, luminance, saturation, alpha").
 		All four values are between 0.0 and 1.0.
 		"""
 		return self.hls() + (self[3]/255.,)
 
-	@ul4c.expose_method
 	def lum(self):
 		"""
 		The luminance value from :meth:`hls`.
@@ -234,7 +228,6 @@ class Color(tuple):
 			channels[3] = a
 		return self.__class__(*channels)
 
-	@ul4c.expose_method
 	def witha(self, a):
 		"""
 		Return a copy of :obj:`self` with the alpha value replaced with :obj:`a`.
@@ -242,7 +235,6 @@ class Color(tuple):
 		(r, g, b, olda) = self
 		return self.__class__(r, g, b, a)
 
-	@ul4c.expose_method
 	def withlum(self, lum):
 		"""
 		Return a copy of :obj:`self` with the luminosity replaced with :obj:`lum`.
@@ -250,7 +242,6 @@ class Color(tuple):
 		(h, l, s, a) = self.hlsa()
 		return self.fromhls(h, lum, s, a)
 
-	@ul4c.expose_method
 	def abslum(self, f):
 		"""
 		Return a copy of :obj:`self` with :obj:`f` added to the luminocity.
@@ -258,7 +249,6 @@ class Color(tuple):
 		(h, l, s, a) = self.hlsa()
 		return self.fromhls(h, l+f, s, a)
 
-	@ul4c.expose_method
 	def rellum(self, f):
 		"""
 		Return a copy of :obj:`self` where the luminocity has been modified:
@@ -275,7 +265,7 @@ class Color(tuple):
 		return self.fromhls(h, l, s, a)
 
 	def __add__(self, other):
-		return self.__class__(0.5*(self[0]+other[0]), 0.5*(self[1]+other[1]), 0.5*(self[2]+other[2]), 255-(255-self[3])*(255-other[3])/255.)
+		raise NotImplementedError
 
 	def __mul__(self, factor):
 		return self.__class__(factor*self[0], factor*self[1], factor*self[2], self[3])
@@ -292,13 +282,52 @@ class Color(tuple):
 	def __mod__(self, other):
 		"""
 		Blends :obj:`self` with the background color :obj:`other` according to the
-		`SVG specification`__
+		`CSS specification`__
 
-		__ http://www.w3.org/TR/2003/REC-SVG11-20030114/masking.html#SimpleAlphaBlending
+		__ https://www.w3.org/TR/2013/WD-compositing-1-20131010/#simplealphacompositing
 		"""
-		sa = self[3]/255.
-		rsa = 1.-sa
-		return self.__class__(self[0]*sa+rsa*other[0], self[1]*sa+rsa*other[1], self[2]*sa+rsa*other[2], 255-rsa*(255-other[3]))
+		# Scale our values to the range [0, 1]
+		rt = self[0]/255.
+		gt = self[1]/255.
+		bt = self[2]/255.
+		at = self[3]/255.
+
+		# Convert to premultiplied alpha
+		rt *= at
+		gt *= at
+		bt *= at
+
+		# Scale other values to the range [0, 1]
+		ro = other[0]/255.
+		go = other[1]/255.
+		bo = other[2]/255.
+		ao = other[3]/255.
+
+		# Convert to premultiplied alpha
+		ro *= ao
+		go *= ao
+		bo *= ao
+
+		# Blend colors
+		rf = rt + ro * (1 - at)
+		gf = gt + go * (1 - at)
+		bf = bt + bo * (1 - at)
+		af = at + ao * (1 - at)
+
+		# Unmultiply alpha
+		if af:
+			rf /= af
+			gf /= af
+			bf /= af
+
+		# Scale back to [0, 255]
+		r = int(255*rf)
+		g = int(255*gf)
+		b = int(255*bf)
+		a = int(255*af)
+
+		# create final color
+		return self.__class__(r, g, b, a)
 
 
 ###
@@ -385,7 +414,12 @@ def screen(c1, c2):
 def mix(*args):
 	"""
 	Calculates a weighted mix of the colors from :obj:`args`. Items in
-	:obj:`args` are either colors or weights.
+	:obj:`args` are either colors or weights. The following example mixes
+	two parts black with one part white::
+
+		>>> from ll import color
+		>>> color.mix(2, color.black, 1, color.white)
+		Color(0x55, 0x55, 0x55)
 	"""
 	channels = [0., 0., 0., 0.]
 	weight = 1.
