@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 # cython: language_level=3, always_allow_keywords=True
 
-## Copyright 2014-2017 by LivingLogic AG, Bayreuth/Germany
-## Copyright 2014-2017 by Walter Dörwald
+## Copyright 2014-2019 by LivingLogic AG, Bayreuth/Germany
+## Copyright 2014-2019 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
@@ -20,6 +20,16 @@ import pytest
 
 @pytest.fixture(scope="module")
 def globals(request):
+	"""
+	Return a ``rul4.Globals`` object with one defined variable: ``connectstring``
+	which will be the value of the environment variable ``LL_ORASQL_TEST_CONNECT``.
+
+	This can be used to create a database connection like this::
+
+		<?code db = globals.oracle(globals.vars.connectstring)?>
+
+	If ``LL_ORASQL_TEST_CONNECT`` is not set ``None`` will be returned instead.
+	"""
 	connectstring = os.environ.get("LL_ORASQL_TEST_CONNECT")
 	if connectstring:
 		return rul4.Globals(vars={"connectstring": connectstring})
@@ -51,7 +61,7 @@ def test_oracle_query(globals):
 			""",
 			whitespace="strip"
 		)
-		assert template.renders(globals=globals) == "1|first|{}|2|second|{}|".format(10000*"first", 10000*"second")
+		assert template.renders(globals=globals) == f"1|first|{10000*'first'}|2|second|{10000*'second'}|"
 
 
 @pytest.mark.db
@@ -93,9 +103,9 @@ def test_oracle_execute_procedure_out(globals):
 					dbms_lob.createtemporary(p_clobarg, true);
 					for i in 1..100000 loop
 						dbms_lob.writeappend(p_clobarg, 3, 'foo');
-					end loop; 
-					p_datearg := to_date('05.10.2014 16:17:18', 'DD.MM.YYYY HH24:MI:SS'); 
-				end; 
+					end loop;
+					p_datearg := to_date('05.10.2014 16:17:18', 'DD.MM.YYYY HH24:MI:SS');
+				end;
 			''')?>
 			<?code vint = db.int()?>
 			<?code vnumber = db.number()?>
@@ -108,4 +118,4 @@ def test_oracle_execute_procedure_out(globals):
 			""",
 			whitespace="strip"
 		)
-		assert template.renders(globals=globals) == "42|42.5|foo|{}|2014-10-05 16:17:18".format(100000*"foo")
+		assert template.renders(globals=globals) == f"42|42.5|foo|{100000*'foo'}|2014-10-05 16:17:18"
